// This is a public domain component for C++Builder 3.0 that fixes the bug in
// TDdeClienConv. The component doesn't work in C++Builder 1.0, but in that
// version the TDdeClientConv component works fine, so no bug fix is required.
//
// In C++Builder 3.0, when you try to call the RequestData member function,
// TDdeClientConv stops with an Access violation error. This new component
// fixes this bug.
//
// This component is based on the idea of Tamas Demjen
// (demjen@usa.net, http://members.xoom.com/demjen)
// The component is written by Heiko Weiss (HeikoWeiss@haas-laser.de)
// Use this component as you like, except you cannot ask money for it, unless
// you make significant modifications on this component.
//
// Usage: This component can be used like TDdeClientConv. There is, however,
//        a difference between TDdeClientConv and TDdeClientConvFix. The
//        character array returned by RequestData must be deallocated with
//        the delete [] operator in TDdeClientConvFix. Here's an example:
//               DdeClientConvFix1->OpenLink();
//               char* s = DdeClientConvFix1->RequestData("getpageinfo");
//               ShowMessage(s);
//               delete[] s;
//        In TDdeClientConv, the returned string had to be deallocated
//        with StrDispose(s);
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "DdeClientConvFix.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
static inline void ValidCtrCheck(TDdeClientConvFix *)
{
   new TDdeClientConvFix(NULL);
}
//---------------------------------------------------------------------------
__fastcall TDdeClientConvFix::TDdeClientConvFix(TComponent* Owner)
   : TDdeClientConv(Owner)
{
}
//---------------------------------------------------------------------------
char* __fastcall TDdeClientConvFix::RequestData(const System::AnsiString Item)
{
   char* result = NULL;
   HDDEDATA hData;
   unsigned long ddeRslt;
   HSZ hItem;
   unsigned char* pData;
   unsigned long len;

   if( Conv == 0 || WaitStat) return result;
   hItem = (HSZ)DdeCreateStringHandle( ddeMgr->DdeInstId, Item.c_str(), CP_WINANSI);

   if(hItem != 0)
   {
      hData = DdeClientTransaction(NULL, 0, (HCONV)Conv, hItem, DdeFmt, XTYP_REQUEST, 10000, &ddeRslt);
      DdeFreeStringHandle(ddeMgr->DdeInstId, hItem);
      if(hData != 0)
      {
         try
         {
            pData = DdeAccessData(hData, &len);
            if(pData != NULL)
            {
               try
               {
                  result = new char[len + 1];
                  memmove(result, pData, len);
                  result[len] = 0;
                  DdeUnaccessData(hData);
               }
               catch(...)
               {
                  DdeUnaccessData(hData);
               }
            }
            DdeFreeDataHandle(hData);
         }
         catch(...)
         {
            DdeFreeDataHandle(hData);
         }
      }
   }
   return result;
}
//---------------------------------------------------------------------------
namespace Ddeclientconvfix
{
   void __fastcall PACKAGE Register()
   {
      TComponentClass classes[1] = {__classid(TDdeClientConvFix)};
      RegisterComponents("Samples", classes, 0);
   }
}
//---------------------------------------------------------------------------
